"""Command line interface for PyHLD."""
from . import __version__
import click
from pathlib import Path
from rich.console import Console
from .commands.design import run_design
from .commands.calculate import run_calculate
from .commands.validate import run_validate
from .commands.graph import run_graph
from .commands.multi2single import run_multi2single
from .commands.poles import run_poles
from .commands.access_points import run_access_points

console = Console()

@click.group()
@click.version_option(version=__version__)
def main():
    """PyHLD - Fiber to the Home High Level Design CLI Tool."""
    pass

@main.command()
@click.option('--input-file', '-i', type=click.Path(exists=True), help='Input data file')
@click.option('--output', '-o', type=click.Path(), help='Output file path')
def design(input_file, output):
    """Create a high-level FTTH design."""
    run_design(input_file, output)

@main.command()
@click.option('--homes', '-h', type=int, help='Number of homes to be served')
@click.option('--area', '-a', type=float, help='Service area in square kilometers')
def calculate(homes, area):
    """Calculate basic FTTH metrics."""
    run_calculate(homes, area)

@main.command()
def validate():
    """Validate design parameters and constraints."""
    run_validate()

@main.command()
@click.option('--input-file', '-i', 
              type=click.Path(exists=True, path_type=Path), 
              required=True,
              help='Input line shapefile path')
@click.option('--delete-duplicates', '-dd',
              is_flag=True,
              help='Delete duplicate geometries')
def graph(input_file, delete_duplicates):
    """Analyze network connectivity from line shapefile."""
    run_graph(input_file, delete_duplicates)

@main.command()
@click.option('--input-file', '-i', 
              type=click.Path(exists=True, path_type=Path), 
              required=True,
              help='Input shapefile path')
@click.option('--output-file', '-o',
              type=click.Path(path_type=Path),
              help='Output shapefile path (optional)')
def multi2single(input_file, output_file):
    """Convert multipart geometries to single part geometries."""
    run_multi2single(input_file, output_file)

@main.command()
@click.option('--segments', '-s', 
              type=click.Path(exists=True, path_type=Path), 
              required=True,
              help='Input segments shapefile path')
@click.option('--poles', '-p',
              type=click.Path(exists=True, path_type=Path),
              required=True,
              help='Input poles shapefile path')
def poles(segments, poles):
    """Validate pole placement for aerial segments."""
    run_poles(segments, poles)

@main.command()
@click.option('--segments', '-s', 
              type=click.Path(exists=True, path_type=Path), 
              required=True,
              help='Input segments shapefile path')
@click.option('--poles', '-p',
              type=click.Path(exists=True, path_type=Path),
              required=True,
              help='Input poles shapefile path')
@click.option('--access-points', '-a',
              type=click.Path(exists=True, path_type=Path),
              required=True,
              help='Input access points shapefile path')
def access_points(segments, poles, access_points):
    """Validate access point placement for underground segments."""
    run_access_points(segments, poles, access_points)

if __name__ == '__main__':
    main()