"""Command to validate access point placement for underground segments."""
from pathlib import Path
import geopandas as gpd
from shapely.geometry import Point
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.panel import Panel
from rich.table import Table

console = Console()

def check_access_points(segments: gpd.GeoDataFrame, 
                       poles: gpd.GeoDataFrame,
                       access_points: gpd.GeoDataFrame,
                       snap_threshold: float = 1.0) -> tuple:
    """Check access point and pole placement for underground segments."""
    # Create/overwrite columns
    segments['access'] = False
    access_points['isolated'] = True  # Default to isolated until proven connected
    
    # Track issues
    missing_endpoints = []
    duplicate_aps = []
    isolated_aps = []
    
    # Process only underground segments
    underground_segments = segments[segments['Type'] == 'Underground']
    
    # Check for duplicate access points
    for idx, ap in access_points.iterrows():
        nearby_aps = access_points[access_points.geometry.distance(ap.geometry) <= snap_threshold]
        if len(nearby_aps) > 1:  # More than itself
            duplicate_aps.append({
                'ap_id': idx,
                'count': len(nearby_aps)
            })
    
    # Check segment endpoints
    for idx, row in underground_segments.iterrows():
        line = row.geometry
        start_point = Point(line.coords[0])
        end_point = Point(line.coords[-1])
        
        # Find poles and APs near endpoints
        start_poles = poles[poles.geometry.distance(start_point) <= snap_threshold]
        end_poles = poles[poles.geometry.distance(end_point) <= snap_threshold]
        start_aps = access_points[access_points.geometry.distance(start_point) <= snap_threshold]
        end_aps = access_points[access_points.geometry.distance(end_point) <= snap_threshold]
        
        # Mark APs as connected if found at endpoints
        if not start_aps.empty:
            access_points.loc[start_aps.index, 'isolated'] = False
        if not end_aps.empty:
            access_points.loc[end_aps.index, 'isolated'] = False
        
        # Check if endpoints have required connections
        has_start = not (start_poles.empty and start_aps.empty)
        has_end = not (end_poles.empty and end_aps.empty)
        
        if not has_start or not has_end:
            missing_endpoints.append({
                'segment_id': idx,
                'start_missing': not has_start,
                'end_missing': not has_end
            })
            segments.loc[idx, 'access'] = False
        else:
            segments.loc[idx, 'access'] = True
    
    # Find isolated access points
    isolated_aps = access_points[access_points['isolated']].index.tolist()
    
    return (segments, access_points, 
            missing_endpoints, duplicate_aps, isolated_aps)

def run_access_points(segments_file: Path, poles_file: Path, 
                     access_points_file: Path) -> None:
    """Validate access point placement."""
    console.print(Panel.fit("🔌 Validating Access Points", title="PyHLD", width=30))
    
    progress = Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        console=console,
    )
    
    with progress:
        # Create tasks
        read_task = progress.add_task("Reading shapefiles", total=1)
        check_task = progress.add_task("Checking access points", total=1, visible=False)
        save_task = progress.add_task("Saving results", total=1, visible=False)
        
        try:
            # Read input files
            segments_gdf = gpd.read_file(segments_file)
            poles_gdf = gpd.read_file(poles_file)
            aps_gdf = gpd.read_file(access_points_file)
            progress.update(read_task, completed=True)
            progress.refresh()
            
            # Check CRS match
            if not (segments_gdf.crs == poles_gdf.crs == aps_gdf.crs):
                console.print("[red]Error: Coordinate systems don't match[/red]")
                return
            
            # Validate access point placement
            progress.update(check_task, visible=True)
            segments_gdf, aps_gdf, missing, duplicates, isolated = check_access_points(
                segments_gdf, poles_gdf, aps_gdf
            )
            progress.update(check_task, completed=True)
            progress.refresh()
            
            # Save results
            progress.update(save_task, visible=True)
            segments_gdf.to_file(segments_file)
            aps_gdf.to_file(access_points_file)
            progress.update(save_task, completed=True)
            progress.stop()
            
            # Report results
            if missing:
                console.print(f"\n[red]Found {len(missing)} underground segments missing connections:[/red]")
                table = Table(title="Missing Connections")
                table.add_column("Segment ID", justify="right")
                table.add_column("Missing At", justify="left")
                
                for m in missing:
                    missing_at = []
                    if m['start_missing']: missing_at.append("Start")
                    if m['end_missing']: missing_at.append("End")
                    table.add_row(str(m['segment_id']), ", ".join(missing_at))
                
                console.print(table)
            
            if duplicates:
                console.print(f"\n[yellow]Warning: Found {len(duplicates)} duplicate access points:[/yellow]")
                table = Table(title="Duplicate Access Points")
                table.add_column("AP ID", justify="right")
                table.add_column("Duplicates", justify="right")
                
                for d in duplicates:
                    table.add_row(str(d['ap_id']), str(d['count']))
                
                console.print(table)
            
            if isolated:
                console.print(f"\n[yellow]Warning: Found {len(isolated)} isolated access points:[/yellow]")
                table = Table(title="Isolated Access Points")
                table.add_column("AP ID", justify="right")
                
                for ap_id in isolated:
                    table.add_row(str(ap_id))
                
                console.print(table)
            
            if not any([missing, duplicates, isolated]):
                console.print("\n[green]✓ All underground segments properly connected[/green]")
            
            # Print summary
            console.print("\n[yellow]Summary:[/yellow]")
            console.print(f"• Total underground segments: {len(segments_gdf[segments_gdf['Type'] == 'Underground'])}")
            console.print(f"• Segments with proper access: {len(segments_gdf[segments_gdf['access']])}")
            console.print(f"• Total access points: {len(aps_gdf)}")
            console.print(f"• Connected access points: {len(aps_gdf[~aps_gdf['isolated']])}")
            
        except Exception as e:
            progress.stop()
            console.print(f"\n[red]Error: {str(e)}[/red]")
            return