"""Command to convert multipart geometries to single part geometries."""
from pathlib import Path
import pandas as pd
import geopandas as gpd
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.panel import Panel

console = Console()

def explode_multiparts(gdf: gpd.GeoDataFrame) -> tuple[gpd.GeoDataFrame, int]:
    """Explode multipart geometries while preserving attributes."""
    # Keep track of original multipart features
    multi_mask = gdf.geometry.apply(lambda geom: geom.geom_type.startswith('Multi'))
    multi_count = multi_mask.sum()
    
    if multi_count == 0:
        return gdf, 0
    
    # Separate single and multipart features
    singles = gdf[~multi_mask].copy()
    multis = gdf[multi_mask].copy()
    
    # Explode multipart features
    exploded = multis.explode(index_parts=True)
    
    # Reset index and drop the multiindex columns
    exploded = exploded.reset_index(drop=True)
    
    # Combine back with single part features
    result = gpd.GeoDataFrame(
        pd.concat([singles, exploded], ignore_index=True),
        crs=gdf.crs
    )
    
    return result, multi_count

def run_multi2single(input_file: Path, output_file: Path = None) -> None:
    """Convert multipart geometries to single part geometries."""
    console.print(Panel.fit("🔨 Converting Multipart to Singlepart", title="PyHLD"))
    
    progress = Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        console=console,
    )
    
    with progress:
        # Create tasks
        read_task = progress.add_task("Reading shapefile", total=1)
        convert_task = progress.add_task("Converting geometries", total=1, visible=False)
        save_task = progress.add_task("Saving results", total=1, visible=False)
        
        try:
            # Read input file
            gdf = gpd.read_file(input_file)
            progress.update(read_task, completed=True)
            progress.refresh()
            
            # Convert multipart to singlepart
            progress.update(convert_task, visible=True)
            result_gdf, multi_count = explode_multiparts(gdf)
            progress.update(convert_task, completed=True)
            progress.refresh()
            
            if multi_count == 0:
                progress.stop()
                console.print("\n[green]No multipart geometries found[/green]")
                return
            
            # Save results
            progress.update(save_task, visible=True)
            output_path = output_file if output_file else input_file
            result_gdf.to_file(output_path)
            progress.update(save_task, completed=True)
            progress.stop()
            
            # Print results
            console.print(f"\n[green]Converted {multi_count} multipart geometries to singlepart[/green]")
            console.print(f"[green]Output saved to:[/green] {output_path}")
            
        except Exception as e:
            progress.stop()
            console.print(f"\n[red]Error: {str(e)}[/red]")
            return