"""Command to validate pole placement for aerial segments."""
from pathlib import Path
import geopandas as gpd
import numpy as np
from shapely.geometry import Point
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.panel import Panel
from rich.table import Table

console = Console()

def check_pole_placement(segments: gpd.GeoDataFrame, poles: gpd.GeoDataFrame, 
                        snap_threshold: float = 1.0) -> tuple[gpd.GeoDataFrame, gpd.GeoDataFrame]:
    """Check pole placement for aerial segments.
    
    Args:
        segments: GeoDataFrame with line segments
        poles: GeoDataFrame with pole points
        snap_threshold: Maximum distance in CRS units to consider a pole snapped
        
    Returns:
        Tuple of (modified segments GDF, modified poles GDF)
    """
    # Create new columns, overwriting if they exist
    segments['poles'] = False
    poles['usage'] = ''
    
    # Track problem segments
    missing_poles = []
    duplicate_poles = []
    
    # Process only aerial segments
    aerial_segments = segments[segments['Type'] == 'Aerial']
    
    for idx, row in aerial_segments.iterrows():
        line = row.geometry
        start_point = Point(line.coords[0])
        end_point = Point(line.coords[-1])
        
        # Find poles near endpoints
        start_poles = poles[poles.geometry.distance(start_point) <= snap_threshold]
        end_poles = poles[poles.geometry.distance(end_point) <= snap_threshold]
        
        # Check for duplicate poles
        if len(start_poles) > 1 or len(end_poles) > 1:
            duplicate_poles.append({
                'segment_id': idx,
                'start_count': len(start_poles),
                'end_count': len(end_poles)
            })
        
        # Check for missing poles
        if len(start_poles) == 0 or len(end_poles) == 0:
            missing_poles.append({
                'segment_id': idx,
                'start_missing': len(start_poles) == 0,
                'end_missing': len(end_poles) == 0
            })
            segments.loc[idx, 'poles'] = False
        else:
            # Mark poles as used for mainline
            poles.loc[start_poles.index, 'usage'] = 'mainline'
            poles.loc[end_poles.index, 'usage'] = 'mainline'
            segments.loc[idx, 'poles'] = True
    
    return segments, poles, missing_poles, duplicate_poles

def run_poles(segments_file: Path, poles_file: Path) -> None:
    """Validate pole placement for aerial segments."""
    console.print(Panel.fit("🏗️  Validating Pole Placement", title="PyHLD", width=30))
    
    progress = Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        console=console,
    )
    
    with progress:
        # Create tasks
        read_task = progress.add_task("Reading shapefiles", total=1)
        check_task = progress.add_task("Checking pole placement", total=1, visible=False)
        save_task = progress.add_task("Saving results", total=1, visible=False)
        
        try:
            # Read input files
            segments_gdf = gpd.read_file(segments_file)
            poles_gdf = gpd.read_file(poles_file)
            progress.update(read_task, completed=True)
            progress.refresh()
            
            # Check CRS match
            if segments_gdf.crs != poles_gdf.crs:
                console.print("[red]Error: Coordinate systems don't match[/red]")
                return
                
            # Validate pole placement
            progress.update(check_task, visible=True)
            segments_gdf, poles_gdf, missing, duplicates = check_pole_placement(
                segments_gdf, poles_gdf
            )
            progress.update(check_task, completed=True)
            progress.refresh()
            
            # Save results
            progress.update(save_task, visible=True)
            segments_gdf.to_file(segments_file)
            poles_gdf.to_file(poles_file)
            progress.update(save_task, completed=True)
            progress.stop()
            
            # Report results
            if missing:
                console.print(f"\n[red]Found {len(missing)} aerial segments missing poles:[/red]")
                table = Table(title="Missing Poles")
                table.add_column("Segment ID", justify="right")
                table.add_column("Missing At", justify="left")
                
                for m in missing:
                    missing_at = []
                    if m['start_missing']: missing_at.append("Start")
                    if m['end_missing']: missing_at.append("End")
                    table.add_row(str(m['segment_id']), ", ".join(missing_at))
                
                console.print(table)
            
            if duplicates:
                console.print(f"\n[yellow]Warning: Found {len(duplicates)} segments with duplicate poles:[/yellow]")
                table = Table(title="Duplicate Poles")
                table.add_column("Segment ID", justify="right")
                table.add_column("Start Count", justify="right")
                table.add_column("End Count", justify="right")
                
                for d in duplicates:
                    table.add_row(
                        str(d['segment_id']),
                        str(d['start_count']),
                        str(d['end_count'])
                    )
                
                console.print(table)
            
            if not missing and not duplicates:
                console.print("\n[green]✓ All aerial segments have correct pole placement[/green]")
            
            # Print summary
            console.print("\n[yellow]Summary:[/yellow]")
            console.print(f"• Total aerial segments: {len(segments_gdf[segments_gdf['Type'] == 'Aerial'])}")
            console.print(f"• Segments with proper poles: {len(segments_gdf[segments_gdf['poles']])}")
            console.print(f"• Poles used as mainline: {len(poles_gdf[poles_gdf['usage'] == 'mainline'])}")
            
        except Exception as e:
            progress.stop()
            console.print(f"\n[red]Error: {str(e)}[/red]")
            return